<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.13
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2021 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Admin\Library;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

class Library extends Templates
{
	public function init()
	{
		// enqueue media
		add_action('admin_enqueue_scripts', [$this, 'register_media']);

		// add the library popup
		add_action('admin_footer', [$this, 'add_library_popup'], 12);
	}

	/**
	 * Adds admin media
	 * 
	 * @return  void
	 */
	public function register_media()
	{
		wp_enqueue_script(
			'fpf-templates',
			FPF_MEDIA_URL . 'admin/js/fpf_templates.js',
			[],
			FPF_VERSION,
			false
		);
	}

	/**
	 * Adds the popup at the footer of the page. Appears when you click the "New" / "Add New" button
	 * 
	 * @param   string  $text
	 * 
	 * @return  void
	 */
	public function add_library_popup()
	{
		// get library layout
		$content = fpframework()->renderer->admin->render('library/tmpl', $this->library_settings, true);
		
		$payload = [
			'id' => $this->modal_id,
			'title' => $this->library_settings['title'],
			'content' => $content,
			'width' => 70,
			'height' => 70
		];
		
		// render a modal with the library layout and return it
        \FPFramework\Helpers\HTML::renderModal($payload);
	}

	/**
	 * Setup ajax requests
	 * 
	 * @return  void
	 */
	public function setupAjax()
	{
		// Get Templates Layout AJAX
		add_action('wp_ajax_fpf_library_get_templates', [$this, 'fpf_library_get_templates']);
		
		// Refresh Templates from Remote and return Layout - AJAX
		add_action('wp_ajax_fpf_library_refresh_templates', [$this, 'fpf_library_refresh_templates']);
		
		// Favorites AJAX
		add_action('wp_ajax_fpf_library_favorites_toggle', [$this, 'fpf_library_favorites_toggle']);
	}

	/**
	 * Checks whether we have the template locally and retrives its layout.
	 * If no local template is found, then retrieves it from remote and returns its layout.
	 * 
	 * @return  string
	 */
	public function fpf_library_get_templates()
	{
		if (!current_user_can('manage_options'))
		{
			return false;
		}
		
		$nonce = isset($_POST['nonce']) ? sanitize_text_field($_POST['nonce']) : '';
		
        // verify nonce
        if (!$verify = wp_verify_nonce($nonce, 'fpf_js_nonce'))
        {
			return false;
		}

		$this->get_templates($this->getList());
	}

	/**
	 * Returns all available templates
	 */
	private function get_templates($templates)
	{
		if (isset($templates->code) || \is_wp_error($templates))
		{
			echo json_encode($templates);
			wp_die();
		}

		$license_type = isset($this->library_settings['license_type']) ? $this->library_settings['license_type'] : 'free';

		$layouts = [];

		// main templates and favorites data
		$free_layouts_payload = [
			'plugin' => $this->library_settings['plugin'],
			'template_use_url' => $this->library_settings['template_use_url'],
			'templates' => $templates,
			'favorites' => $this->getFavorites()
		];
		// Add the license type only if we are a free user, otherwise show all templates
		if ($license_type != 'pro')
		{
			$free_layouts_payload['license_type'] = 'free';
		}
		$layouts['main'] = fpframework()->renderer->admin->render('library/items_list', $free_layouts_payload, true);

		// If we are a free user, also load the pro templates separately
		if ($license_type == 'free')
		{
			// pro templates and favorites data
			$layouts['pro'] = fpframework()->renderer->admin->render('library/items_list', [
				'plugin' => $this->library_settings['plugin'],
				'template_use_url' => $this->library_settings['template_use_url'],
				'templates' => $templates,
				'favorites' => $this->getFavorites(),
				'license_type' => 'pro'
			], true);
		}

		echo json_encode($layouts);
		wp_die();
	}

	/**
	 * Retrieve remote templates, store them locally and return new layout.
	 * 
	 * @return  string
	 */
	public function fpf_library_refresh_templates()
	{
		if (!current_user_can('manage_options'))
		{
			return false;
		}
		
		$nonce = isset($_POST['nonce']) ? sanitize_text_field($_POST['nonce']) : '';
		
        // verify nonce
        if (!$verify = wp_verify_nonce($nonce, 'fpf_js_nonce'))
        {
			return false;
		}

		$this->get_templates($this->getRemoteTemplatesAndStore());
	}

	/**
	 * Handles AJAX Library favorite toggle
	 * 
	 * @return  string
	 */
	public function fpf_library_favorites_toggle()
	{
		if (!current_user_can('manage_options'))
		{
			return false;
		}
		
		$nonce = isset($_POST['nonce']) ? sanitize_text_field($_POST['nonce']) : '';
		
        // verify nonce
        if (!$verify = wp_verify_nonce($nonce, 'fpf_js_nonce'))
        {
			return false;
		}
		
		$template_id = isset($_POST['template_id']) ? sanitize_text_field($_POST['template_id']) : '';

		if (empty($template_id))
		{
			return false;
		}
		
		$this->addOrRemoveFavorite($template_id);
		
		echo json_encode($this->getFavorites());
		wp_die();
	}
}