<?php
/**
 * @package         FirePlugins Framework
 * @version         1.0.13
 * 
 * @author          FirePlugins <info@fireplugins.com>
 * @link            https://www.fireplugins.com
 * @copyright       Copyright © 2021 FirePlugins All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
*/

namespace FPFramework\Base;

if (!defined('ABSPATH'))
{
	exit; // Exit if accessed directly.
}

class Assignments
{   
    /**
     *  Factory object 
     * 
     *  @var \FPFramework\Base\Factory
     */
    protected $factory;

    /**
     *  Constructor
     */
    public function __construct($factory = null)
    {
        if (!$factory)
        {
            $factory = new \FPFramework\Base\Factory();
        }

        $this->factory = $factory;        
    }

    /**
	 *  Check all Assignments
	 *
	 *  @param   array|object   $assignments_info   Array/Object containing assignment info
	 *  @param   string         $match_method       The matching method (and|or) - Deprecated
	 *  @param   bool           $debug              Set to true to request additional debug information about assignments
     * 
	 *  @return  bool|array                         True if check passes. If $debug is set to true an array will be returned with
     *                                              the result in the first element and debug info in the second.
	 */
	public function passAll($assignments_info, $match_method = 'and', $debug = false)
	{
        if (!$assignments_info)
        {
            return true;
        }

        // convert $assignments_info parameter from object (used by existing extensions) to array
        if (is_object($assignments_info))
        {
            $assignments_info = $this->prepareAssignmentsFromObject($assignments_info, $match_method);
        }

        // prepare assignment data
        $assignments = $this->prepareAssignments($assignments_info);

        $debug_info = [];
        if ($debug)
        {
            $debug_info = $this->generateDebugInfo($assignments);
        }

        // return true if no assignments are given
        if (empty($assignments))
        {
            return $debug ? [true, $debug_info] : true;
        }

        $pass = false;

        foreach ($assignments as $group)
        {
            // Pass all assignments in the group
            if ($pass = $this->passAnd($group))
            {
                break;
            }
        }

        return $debug ? [$pass, $debug_info] : $pass;
    }

    /**
     * Check if all of the given assignments passes the check
     *
     * @param   array   $assignments       The assignments array to check
     *
     * @return  bool
     */
    private function passAnd($assignments)
    {
        if (!is_array($assignments) || count($assignments) == 0)
        {
            return;
        }

        foreach ($assignments as $assignment)
        {
            if (is_null($assignment) || !\property_exists($assignment, 'class') || is_null($assignment->class))
            {
                // OnOtherBox exists in FB, it searches for it in FPF
                // May need to detect where the Assignment exists in order to check it in correct namespace
                continue;
                // return;
            }

            $assignmentInstance = new $assignment->class($assignment->options, $this->factory);
            $pass = $this->passStateCheck($assignmentInstance->pass(), $assignment->options->assignment_state);

            // Fail if any of the assignments doesn't pass the check.
            if (!$pass)
            {
                return false;
            }
        }

        return true;
    }
   
    /**
     *  Checks if an assignment exists
     *
     *  @param  string $assignment Assignment class name or alias
     *  @return bool
     */
    public function exists($assignment)
    {
        if (!$assignment)
        {
            return false;
        }

        $assignment = strtolower($assignment);
        $assignment = str_replace('__', '/', $assignment);
        $assignment = ucfirst($assignment);

        $file = dirname(__FILE__) . '/Assignments/' . $assignment . '.php';

        if (!file_exists($file))
        {
            return false;
        }

        return true;
    }

    /**
     *  Returns the classname for a given assignment alias
     *
     *  @param  string       $alias
     *  @return string|void
     */
    public function aliasToClassname($alias)
    {
        $alias = strtolower($alias);
        $alias = str_replace('__', '\\', $alias);
        $alias = ucwords($alias, '\\');
        
        return $alias;
    }

    /**
    *  Assignment pass check based on the assignment state
    *
    *  @param   boolean  $pass        
    *  @param   string   $assignment_state  The assignment state
    *
    *  @return  boolean
    */
    private function passStateCheck($pass = true, $assignment_state = null)
    {
        $assignment_state = $assignment_state ?: $this->assignment;
        return $pass ? ($assignment_state == 'include') : ($assignment_state == 'exclude');
    }

    /**
     *  Checks and prepares the given array of assignment information
     * 
     *  @param   array $assignments_info
     *  @return  array
     */
    protected function prepareAssignments($assignments_info)
    {
        $assignments = [];

        foreach ($assignments_info as $group)
        {
            if (empty($group)) 
            {
                continue;
            }

            $newGroup = [];

            foreach ($group as $a)
            {
                // check if the object has the required properties
                if (!is_object($a) ||!isset($a->alias) || !isset($a->value) || !isset($a->assignment_state))
                {
                    continue;
                }

                $assignment = new \stdClass();

                // check if the assignment type exists
                if (!$this->exists($a->alias) || !$this->setTypeParams($assignment, $this->aliasToClassname($a->alias)))
                {
                    $assignment->class = null;
                }

                $assignment->options = (object) array(
                    'alias'             => $a->alias,
                    'selection'         => $a->value,
                    'params'            => isset($a->params) ? $a->params : new \stdClass(),
                    'assignment_state'  => $this->getAssignmentState($a->assignment_state)
                );

                $newGroup[] = $assignment;
            }

            $assignments[] = $newGroup;
        }

        return $assignments;
    }

    /**
     *  Converts an object of assignment information to an array of groups
     *  Used by existing extensions
     * 
     *  @param  object $assignments_info
     *  @param  string $matching_method
     * 
     *  @return array of objects
     */
    public function prepareAssignmentsFromObject($assignments_info, $match_method)
    {
        if (!isset($assignments_info->params))
        {
            return [];
        }
        
        $params = json_decode($assignments_info->params);

        if (!is_object($params))
		{
			return [];
        }

        if (!isset($params->assignments))
        {
            return [];
        }

        $assignments_info = [];
        
        foreach ($params->assignments as $key => $assignment)
        {
            if (!isset($assignment->selection))
            {
                continue;
            }

            if (!in_array((int) $assignment->selection, [1, 2]))
            {
                continue;
            }

            // Discover assignment params
            $assignment_params = new \stdClass();
            foreach ($assignment as $_key => $value)
            {
                if (strpos($_key, 'param_') === 0)
                {
                    $_key = str_replace('param_', '', $_key);
                    $assignment_params->$_key = $value;
                }
            }

            $key = str_replace('assign_', '', $key);
            $assignments_info[] = (object) array(
                'alias'             => $key,
                'assignment_state'  => $this->getAssignmentState($assignment->selection),
                'value'             => isset($assignment->list) ? $assignment->list : [],
                'params'            => $assignment_params
            );

        }

        if ($match_method === 'or')
        {
            // each assignemnt belongs to a separate group
            $res = [];
            foreach($assignments_info as $assignment)
            {
                $res[] = [$assignment];
            }
            return $res;
        }
        else 
        {
            // every assignment belongs to the same group
            return [$assignments_info];
        }
    }

    /**
	 *  Returns assignment's state by ID
	 *  1: Include
	 *  2: Exclude
	 *  3, -1: None
	 *
	 *  @param   integer  $state_id     Assignment's state ID
	 *
	 *  @return  string                 Assignment's state name
	 */
	private function getAssignmentState($state_id)
	{
		switch ($state_id)
		{
			case 1:
			case 'include':
				$assignment_state = 'include';
				break;
			case 2:
			case 'exclude':
				$assignment_state = 'exclude';
				break;
			case 3:
			case -1:
			case 'none':
				$assignment_state = 'none';
				break;
			default:
				$assignment_state = 'all';
				break;
		}

		return $assignment_state;
    }
    
    /**
	 *  Sets proper assignment class and method name
	 *
	 *  @param   object  &$assignment  The assignment object
	 *  @param   string  $type         The assignment type
	 *
	 *  @return  bool                   True if the class and method exist, false otherwise 
	 */
	public function setTypeParams(&$assignment, $type = '')
	{
        $class  = __NAMESPACE__ . '\\Assignments\\' . $type;
        if (!class_exists($class))
        {
            return false;
        }
        
        $assignment->class  = $class;

        return true;
    }
    
    /**
     *  Checks assignments and returns debug information
     * 
     *  @param  array $assignments
     * 
     *  @return array 
     */
    protected function generateDebugInfo($assignments)
    {
        $debug_info = [];
        foreach ($assignments as $group)
        {
            $debugGroup = [];
            foreach($group as $assignment)
            {
                if (!property_exists($assignment, 'class') || is_null($assignment->class))
                {
                    $assignment->pass = null;
                    $assignment->name = 'Unknown Assignment';
                }
                else
                {
                    $inst   = new $assignment->class($assignment->options, $this->factory);
                    $passed = $inst->pass();
                    $assignment->pass = $this->passStateCheck(
                        $passed,
                        $assignment->options->assignment_state
                    );
                    $assignment->value  = $inst->value();
                    $assignment->name   = \preg_replace('/.*\\\\(.*)$/', "$1", $assignment->class);
                }
                $debugGroup[] = $assignment;
            }
            $debug_info[] = $debugGroup;
        }

        return $debug_info;
    }
}